/* Copyright (C) 2002, by Sandor Fazekas and Janos Torok */
/* Last modified: $Date: 2003/02/07 15:13:11 $ $Revision: 1.1 $ */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "d2mglobal.h"
#include "io.h"

typedef struct Param_ {
  const char *name;	/* parameter name */
  int type;		/* type */
  void *data;		/* where data is to be stored */
  int is_req;		/* if it is required */
} Param;

/* types */
#define PARI	1	/* int */
#define PARD	2	/* double */
#define PARS	3	/* string */
#define LNUM	4	/* light num */
#define LPAR	5	/* light struct */
#define COLS	6	/* background */

static Param param_list[] = {
  { "filename",		PARS, &io.filename,	1 },
  { "filetype",		PARI, &gl.filetype,	0 },
  { "tmpdir",		PARS, &gl.tmpdir,	0 },
  { "outfile",		PARS, &gl.outfile,	0 },
  { "pattern",		PARS, &gl.pattern,	0 },
  { "framerate",	PARD, &gl.framerate,	0 },
  { "framerepeat",	PARI, &gl.framerepeat,	0 },
  { "quadres",		PARI, &gl.quadres,	0 },
  { "quality",		PARI, &gl.quality,	0 },
  { "dx",		PARD, &gl.x,		0 },
  { "dy",		PARD, &gl.y,		0 },
  { "dz",		PARD, &gl.z,		0 },
  { "factor",		PARD, &gl.f,		0 },
  { "phix",		PARD, &gl.phix,		0 },
  { "phiy",		PARD, &gl.phiy,		0 },
  { "phiz",		PARD, &gl.phiz,		0 },
  { "dx/dt",		PARD, &gl.dx,		0 },
  { "dy/dt",		PARD, &gl.dy,		0 },
  { "dz/dt",		PARD, &gl.dz,		0 },
  { "phix/dt",		PARD, &gl.dphix,	0 },
  { "phiy/dt",		PARD, &gl.dphiy,	0 },
  { "phiz/dt",		PARD, &gl.dphiz,	0 },
  { "startframe",	PARI, &gl.startframe,	0 },
  { "endframe",		PARI, &gl.endframe,	0 },
  { "frameskip",	PARI, &gl.frameskip,	0 },
  { "width",		PARI, &gl.width,	0 },
  { "height",		PARI, &gl.height,	0 },
  { "bg",		COLS, &gl.bg,		0 },
  { "playdir",		PARI, &gl.playdir,	0 },
  { "lightnum",		LNUM, &gl.lightnum,	0 },
  { "param",		LPAR, &gl.light,	0 }
};

#define PARAM_NUM sizeof(param_list)/sizeof(Param)

int Which_Param(char *p)
{
  int a;

  for (a=0;a<PARAM_NUM;a++) {
    if (strcmp(p,param_list[a].name)==0) {
      return(a);
    }
  }
  return(-1);
}

void Read_Params(FILE *in)
{
  int ppos;
  int a,b;
  char *p,*pp;
  double *dp;
  struct lightstr *tmplight;

  while (!io.eof(in)) {
    io.getline(in);
    p=io.buff;
    /* remove '\n at the end */
    if (p[strlen(p)-1]=='\n') p[strlen(p)-1]='\0';
    /* remove whitespaces */
    while (isspace(*p)) p++;
    if (*p!='#') {	/* comment */
      pp=p;
      while (!isspace(*pp)) pp++;
      *pp='\0';
      ppos=Which_Param(p);
      if (ppos>=0) {
	p=pp+1;
	while ((isspace(*p)) || (*p=='=')) p++;
	/* now set value */
	switch (param_list[ppos].type) {
	  case PARI:	/* int */
	    *((int *)(param_list[ppos].data))=atoi(p);
	    param_list[ppos].is_req=0;
	    break;
	  case PARD:	/* double */
	    *((double *)(param_list[ppos].data))=atof(p);
	    param_list[ppos].is_req=0;
	    break;
	  case PARS:	/* string no static is assumed */
	    if (param_list[ppos].is_req) {
	      free(*((char **)(param_list[ppos].data)));
	    }
	    *((char **)(param_list[ppos].data))=strdup(p);
	    param_list[ppos].is_req=0;
	    break;
	  case LNUM:	/* light numer */
	    /* it has to be treated seperately because of allocation */
	    a=atoi(p);
	    if (a>gl.lightnum) {
	      tmplight=(struct lightstr *) calloc(sizeof(struct lightstr),a);
	      memcpy(tmplight,gl.light,gl.lightnum*sizeof(struct lightstr));
	      free(gl.light);
	      gl.light=tmplight;
	    }
	    gl.lightnum=a;
	    param_list[ppos].is_req=0;
	    break;
	  case LPAR:	/* light parameter */
	    /* first we read light and parameter number */
	    sscanf(p,"%d %d",&a,&b);
	    if ((a>gl.lightnum) || (b>4)) {
	      fprintf(stderr,"dat2mpg warning: error with light data\n");
	      fprintf(stderr,"> %s\n",p);
	    } else {
	      sscanf(p,"%*d %*d %e %e %e %e",&gl.light[a].colors[b][0],
		    &gl.light[a].colors[b][1],&gl.light[a].colors[b][2],
		    &gl.light[a].colors[b][3]);
	    }
	    param_list[ppos].is_req=0;
	    break;
	  case COLS:
	    dp=(double *)(param_list[ppos].data);
	    sscanf(p,"%le %le %le %le",dp,dp+1,dp+2,dp+3);
	    param_list[ppos].is_req=0;
	    break;
	  default:
	    fprintf(stderr,"dat2mpg error: unknown data type\n");
	    exit(1);
	}
      } else {
	fprintf(stderr,"dat2mpg warning: unknown parameter:\n");
	fprintf(stderr,"> %s\n",p);
      }
    }
  }
  for (a=0;a<PARAM_NUM;a++) {
    if (param_list[a].is_req) {
      fprintf(stderr,"dat2mpg error: parameter %s was not initialized\n",
	  param_list[a].name);
      exit(1);
    }
  }
}
